﻿define("fotowareintegration/editors/FotoWareThumbnailSelector", [
    // dojo
    "dojo/_base/declare",
    "dojo/when",
    "dojo/on",
    "dojo/topic",
    "dojo/dom-class",
    "dojo/ready",
    "dojo/_base/lang",
    // dijit
    "dijit/focus",
    "dijit/ProgressBar",
    // epi
    "epi/Url",
    "epi/routes",
    "epi/shell/dnd/Target",
    "epi/shell/DialogService",
    "epi/shell/XhrWrapper",
    // epi-cms
    "epi-cms/core/ContentReference",
    "epi-cms/widget/MediaSelector",
    "epi-cms/widget/ThumbnailSelector",
    "epi-cms/command/UploadContent",
    "epi-cms/widget/UploadUtil",
    "epi-cms/widget/viewmodel/MultipleFileUploadViewModel",
    "epi-cms/widget/LocalFolderUploader",
    "dojo/text!./templates/FotoWareThumbnailSelector.html",
    "epi-cms/legacy/LegacyDialogPopup",
    "epi/i18n!epi/cms/nls/episerver.cms.widget.thumbnailselector"
], function (
    // dojo
    declare,
    when,
    on,
    topic,
    domClass,
    ready,
    lang,
    // dijit
    focusManager,
    ProgressBar,
    // epi
    Url,
    routes,
    Target,
    dialogService,
    XhrWrapper,
    // epi-cms
    ContentReference,
    MediaSelector,
    ThumbnailSelector,
    UploadContent,
    UploadUtil,
    MultipleFileUploadViewModel,
    LocalFolderUploader,
    template,
    LegacyDialogPopup,
    resources
) {
        var xhr = new XhrWrapper();
        return declare([ThumbnailSelector], {
            // summary:
            //      Represents the widget to select ContentReference.
            // tags:
            //      internal

            resources: resources,
            templateString: template,

            loaded: false,
            uploadDisabledMessage: "Drag and drop feature has been disabled, please use Fotoware.",
            fotowarepath: null,
            fotowareversion: null,
            fotowareiconpath: null,
            fotowaretenanturl: null,
            fotowaremodulepath: null,
            fotowarepluginWindow: null,
            fwWin: null,
            pluginurl: null,
            timer: null,
            disableMediaSelector: null,
            listenerBound: false,
            //      List of file extensions that may be dropped onto the editor
            allowedExtensions: [],
            postCreate: function () {

                ready(setTimeout(lang.hitch(this, function () {//not optimal with timeout, but else not working with _setValue when used on init
                    this.loaded = true;
                }), 100));

                this.allowedExtensions = this.fotowareallowedfiletypes.replace(".", "").replace(/,/gi, "").split('.');

                this.inherited(arguments);
            },
            getFetchCf: function (url, data) {
                var self = this;
                return xhr.xhrGet({
                    url: url,
                    content: data,
                    failOk: true,
                    error: function (response) {
                        self._dialogService.alert(response.xhr.statusText);
                    }
                });
            },
            // Custom function for Fotoware integration to trigger opening of dialog.
            _onFotoWareButtonClick: function () {

                this.pluginurl = this.fotowarepath + "ClientResources/plugin/fotoware.html?fotowareBaseUrl=" +
                    this.fotowaretenanturl +
                    "/fotoweb/widgets/selection&insertMultiple=false&cn=" + this.fotowarecookiename + "&";

                this.fwWin = new LegacyDialogPopup({
                    url: this.pluginurl + Math.random(),
                    onCallback: null,
                    dialogArguments: {},
                    dialogTitle: "Fotoware Selection Widget v" + this.fotowareversion,
                    autoFit: false,
                    showCancelButton: true,
                    showLoadingOverlay: false,
                    features: { width: 1200, height: 900 }
                });
                this.fwWin.actionContainerNode.hidden = true;
                this.fwWin.show();

                if (!this.listenerBound) {
                    this._listener = this._listener.bind(this);
                    window.fotowarepluginWindow = this.fwWin;
                    window.addEventListener("message", this._listener, true);
                    this.listenerBound = true;
                }

                this.timer = setInterval(lang.hitch(this,
                    function () {
                        if (!this.fwWin.open) {
                            window.removeEventListener("message", this._listener, true);
                            this.listenerBound = false;
                            clearInterval(this.timer);
                            return;
                        }
                    }),
                    2000);

            },
            _listener: function (event) {
                   
                if (event.data == "loggedinSuccess") {
                    this.fwWin.location = this.pluginurl + Math.random();
                    return;
                }
                else if (event.data.event === 'selectionWidgetCancel') {
                    this.fwWin.hide();
                    //remove listener
                    window.removeEventListener("message", this._listener, true);
                    this.listenerBound = false;
                }
                else if (event.data.event === 'assetSelected') {
                    if (this.fotowareenableexport==="True") {
                        this.fwWin.wrapper.containerIframe.src = this.fotowarepath +
                            "ClientResources/plugin/fotoware.html?fotowareBaseUrl=" +
                            this.fotowaretenanturl +
                            "/fotoweb/widgets/publish&i=" +
                            encodeURIComponent(event.data.asset.href) +
                            "&" + Math.random() +
                            "&cn=" + this.fotowarecookiename +
                            "&pass=true&" + this.fotowareextraquerystring;
                        return;
                    } else {
                        //get the default image
                        this.fwWin.hide();

                        this.assetChoosen(event.data.asset.href);

                        //remove listener
                        window.removeEventListener("message", this._listener, true);
                        this.listenerBound = false;
                    }

                }
                else if (event.data.event === 'assetExported') {
                    this.fwWin.hide ();
                    this.assetChoosenExport(event.data.export.export.source.replace(this.fotowaretenanturl, ""), event.data.export.export.image.normal.replace(this.fotowaretenanturl, ""));

                    //remove listener
                    window.removeEventListener("message", this._listener, true);
                    this.listenerBound = false;

                }
                //there are more events sent, like frame load, page load, focus
                    
                
            },
            assetChoosen: function (assethref){
                var self = this;
                when(self.getFetchCf(
                        self.fotowaremodulepath + "authentication/fetchcf",
                        {
                            hostcontent: "",
                            image: assethref
                        }),
                    function (data) {
                        self._setValueAndFireOnChange(data + "");
                    });
            },
            assetChoosenExport: function (assethref, exporturl) {
                var self = this;
                when(self.getFetchCf(
                        self.fotowaremodulepath + "authentication/fetchcf",
                        {
                            hostcontent: "",
                            image: assethref,
                            exporturl: exporturl
                        }),
                    function (data) {
                        self._setValueAndFireOnChange(data + "");
                    });
            },
            //if you want to get other thumbnail from fotoware for types that episerver do not have thumbnails
            //_updateDisplayNode: function (content) {
            //    this.inherited(arguments);
            //    if (content) {
            //        this.thumbnail.src = this._getThumbnailUrl(content).toString();
            //        //custom thumbnail
            //        if (!content.name.match(/.(jpg|jpeg|png|gif|bmp|jpe)$/i)) {
            //            this.thumbnail.src = this.fotowaremodulepath + "authentication/GenerateThumb?contentid=" + content.contentLink;
            //            this.thumbnail.style = "object-fit:contain;";
            //        }
            //    }
            //    this.stateNode.title = content ? content.name : "";
            //},
            //buildUrl: function (url, assethref) {

            //    var baseUrl = url + '?i=' + assethref;

            //    baseUrl = baseUrl + this.getUrlPart('&w=', this.defaultWidth);
            //    baseUrl = baseUrl + this.getUrlPart('&h=', this.defaultHeight);
            //    baseUrl = baseUrl + this.getUrlPart('&p=', this.defaultPreset);

            //    return baseUrl;
            //},
            getUrlPart: function (key, value) {
                if (typeof (value) != 'undefined' && value != "") {
                    return key + value;
                }
                return "";
            },

            _setValueAttr: function (value) {
                if (this.disableMediaSelector==="True" && this.loaded && value != null) {
                    this._dialogService.alert(this.uploadDisabledMessage);
                    return;
                }
                this.inherited(arguments);
            },

            _onButtonClick: function () {
                if (this.readOnly) {
                    return;
                }

                if (this.disableMediaSelector==="True") {
                    this._onFotoWareButtonClick();
                    return;
                }


                this.inherited(arguments);
            }
        });
    });
